'' HTTP Client Routines
'' --------------------
'' (c) 2009. Harrison Pham <harrison@harrisonpham.com>.
''
'' - Assumes that you have already started and inited driver_socket
'' - Note: All methods may abort on a socket error, so you must catch them with a \

CON
  CONNECTTIMEOUT = 2000         ' connection timeout in milliseconds
  RETRIES        = 5            ' number of retries before aborting
  RETRYDELAY     = 250          ' retry delay in milliseconds

CON
  ERRSOCKET      = -1
  ERRUNREACHABLE = -2
  ERRNOMOREDATA  = -3

OBJ
  sock  : "api_telnet_serial"
  str   : "util_strings"
  num   : "Numbers"

VAR
  byte buff[33]                 ' temporary buffer

  long clen                     ' content length (bytes)
  long cpos                     ' content position (byte index)

PUB disconnect

  sock.close

PUB startGETRequest(ipaddr, port, strHost, strUri)
  '' Initiates a HTTP GET request
  ''
  '' ipaddr = IPv4 Address (long)
  '' port   = Port number (word)
  '' strUri = Request URI (stringz)
  ''          ex: string("/blah/mypage.php?var=2323")
  ''
  '' Returns: Content length if successful, otherwise aborts
  
  if _httpConnect(ipaddr, port) == 0
    ' connected
    sock.str(string("GET "))
    sock.str(strUri)
    sock.str(string(" HTTP/1.0",13,10,"Host: "))
    sock.str(strHost)
    sock.str(string(13,10,"User-Agent: PropTCP",13,10,"Connection: close",13,10,13,10))

  ' get content length
  return _getContentLength  

PUB startPOSTRequest(ipaddr, port, strHost, strUri)
  '' Initiates a HTTP POST

  if _httpConnect(ipaddr, port) == 0
    ' connected
    sock.str(string("POST "))
    sock.str(strUri)
    sock.str(string(" HTTP/1.0",13,10,"Host: "))
    sock.str(strHost)
    sock.str(string(13,10,"User-Agent: PropTCP",13,10,"Connection: close",13,10,"Content-Type: multipart/form-data; boundary="))
    sock.str(@mimeboundary + 2)
    sock.str(string(13,10,"Content-Length: 0",13,10,"Transfer-Encoding: chunked",13,10,13,10))

PUB startPOSTFile(strFieldName, strFileName)

  _chunkHeader(strsize(@mimeboundary) + strsize(strFieldName) + strsize(strFileName) + 133)
  
  sock.str(@mimeboundary)
  sock.str(string(13,10,"Content-Disposition: form-data; name=",$22))
  sock.str(strFieldName)
  sock.str(string($22,"; filename=",$22))
  sock.str(strFileName)
  sock.str(string($22,13,10,"Content-Type: application/octet-stream",13,10,"Content-Transfer-Encoding: binary",13,10,13,10))

  _chunkFooter

PUB appendPOSTFileData(ptrData, len)

  _chunkHeader(len)
  sock.txdata(ptrData, len)
  _chunkFooter

PUB endPOSTFile

  _chunkHeader(2)
  sock.str(@crlf)
  _chunkFooter

PUB endPOSTRequest

  _chunkHeader(strsize(@mimeboundary) + 4)
  sock.str(@mimeboundary)
  sock.str(string("--",13,10))
  _chunkFooter

  _chunkHeader(0)
  _chunkFooter

  'sock.close
  return _getContentLength

PUB getResponseChunk(ptr, len) | rxlen
  '' Gets the response content

  if cpos => clen
    return ERRNOMOREDATA
  
  rxlen := sock.rxdata(ptr, len)

  if rxlen > 0
    cpos += rxlen
  
  return rxlen

PRI _chunkHeader(len)
  sock.hex(len, 4)
  sock.str(@crlf)

PRI _chunkFooter
  sock.str(@crlf)

PRI _getContentLength | i

  clen := -1
  cpos := 0
  repeat while _readLine > 0
    if (i := str.indexOf(@buff, string("Content-Length:"))) > -1
      clen := num.FromStr(@buff[i + 15], num#DEC)

  return clen
  
PRI _readLine | i, ch
  repeat i from 0 to 32
    ch := \sock.rx
    if ch == 10
      ch := \sock.rx
    if ch == -1 or ch == 13
      quit
    buff[i] := ch

  buff[i] := 0

  return i

PRI _httpConnect(ipaddr, port)
  ' performs http connection

  repeat RETRIES
    sock.connect(ipaddr, port)
    sock.resetBuffers
    if sock.waitConnectTimeout(CONNECTTIMEOUT)
      return 0
    \sock.close
    delay_ms(RETRYDELAY)

  \sock.close
  abort ERRUNREACHABLE

PRI delay_ms(Duration)
  waitcnt((clkfreq / 1_000 * Duration - 3932) + cnt)

DAT

crlf          byte      13,10,0
mimeboundary  byte      "-------------------------------18788734234",0
  